// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#ifndef NPRV_L1_L2_RADIO
#define NPRV_L1_L2_RADIO

#include <stdint.h>
#include <cstdint>

uint32_t NPRV_compute_slot_duration(int channel, int frame_length);

void NPRV_L1L2_print_TX_frames(void);

int NPRV_decide_tx_new_frame(unsigned char* raw_data, int channel, int* modulation, uint32_t T_end_burst, unsigned char uslot_count);

int NPRV_get_null_TXr_frame(unsigned char* data, int approx_size);

void NPRV_print_pseudorand_err(void);

void NPRV_code_checksum(unsigned char* data, int size);

int NPRV_check_checksum(unsigned char* data, int size);

void NPRV_code_header(unsigned char* data_wo_FEC,
	int modulation, int top_sync, unsigned int length, unsigned int third_byte, unsigned int TX_ID,
	unsigned char burst_counter, unsigned char channel, unsigned char slot_counter);

//int NPRV_rx_TOTAL_header_interp(
//	unsigned char* data, int* modulation,
//	int* top_sync, unsigned int* length, unsigned int* TDMA_byte, unsigned int* TX_ID,
//	unsigned char* burst_counter, unsigned char* channel, unsigned char* slot_counter);

int NPRV_rx_TOTAL_header_interp(
	unsigned char* data, int* modulation,
	int* top_sync, unsigned int* length, unsigned char* TDMA_byte, unsigned int* TX_ID,
	unsigned char* burst_counter, unsigned char* channel, unsigned char* slot_counter);

int NPRV_rx_RT_header_interp(unsigned char* data, int* modulation, int* length);

void NPRV_L1L2_print_RX_frame_count(void);

//int NPRV_raw_frame_interp(
//	unsigned char* raw_frame, int size, uint32_t sync_time, float amplitude, float freq_offset, int channel);
int NPRV_raw_frame_interp(
	unsigned char* raw_frame,
	int size,
	uint32_t sync_time,
	float amplitude,
	float freq_offset,
	int channel,
	int BER_header,
	int BER_DataField,
	int byte_size);

#endif