// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#include "pch.h"
#include <stdlib.h>     /* srand, rand */
#include <SFML/System.hpp>
#include "NPRV_glob_variables.h"
#include "NPRV_SDR_if.h"
#include "NPRV_NVRAM.h"
#include "NPRV_display.h"

#pragma warning(disable: 4996)

//G_NPRV_my_TimeAdvance int32_t

#define NPRV_NVRAM_TA_INIT 340000

uint16_t NVRAM_posX_constel = 0; 
uint16_t NVRAM_posY_constel = 0;
uint16_t NVRAM_posX_eye = 0;
uint16_t NVRAM_posY_eye = 0;

int NPRV_init_NVRAM(void) {
	uint32_t rand_nb_loc;
	uint8_t rand_nb_char[6];
	int i;
	sf::sleep(sf::milliseconds(2000));
	for (i = 0; i < 1000; i++) {
		rand();
	}
	//srand(time(NULL));
	rand_nb_loc = rand();
	rand_nb_char[0] = (rand_nb_loc & 0x0000FF00) >> 8;
	rand_nb_char[1] = (rand_nb_loc & 0x000000FF);
	//srand(time(NULL) + 32);
	rand_nb_loc = rand();
	rand_nb_char[2] = (rand_nb_loc & 0x0000FF00) >> 8;
	rand_nb_char[3] = (rand_nb_loc & 0x000000FF);
	printf("RAND generator: %02X %02X %02X %02X\r\n", rand_nb_char[0], rand_nb_char[1], rand_nb_char[2], rand_nb_char[3]);
	for (i = 0; i < 4; i++) {
		G_NPRV_my_random_nb[i] = rand_nb_char[i];
	}
	CONF_modem_MAC[4] = rand_nb_char[2];
	CONF_modem_MAC[5] = rand_nb_char[3];

	NPRV_SDR_init_gain(1);
	G_NPRV_my_TimeAdvance = NPRV_NVRAM_TA_INIT;
	NVRAM_posX_constel = 10;
	NVRAM_posY_constel = 60;
	NVRAM_posX_eye = 200;
	NVRAM_posY_eye = 60;
	return 1;
}

int NPRV_read_NVRAM(void) {
	FILE* fid_in;
	uint8_t raw_data[60];
	uint8_t ref_data[20] = { "NPRV_nvram001:" };
	int32_t loc_TA, loc_gain_RX, loc_gain_TX;
	uint8_t loc_rand_nb[10];
	uint16_t posX_constel, posY_constel, posX_eye, posY_eye;
	int16_t offset_freqTX_tuned, offset_freqTX_conf, offset_freqRX_tuned, offset_freqRX_conf;
	int i;
	int match = 1;
	int config_OK = 1;
	fid_in = fopen("prog_data/NPRVSAT.nvm", "rb");
	//fread
	if (fid_in != 0) {
		fread(raw_data, sizeof(uint8_t), 38, fid_in);//30 38
	}
	else {
		config_OK = 0;
	}
	for (i = 0; i < 14; i++) {
		if (raw_data[i] != ref_data[i]) { match = 0; }
	}
	for (i = 0; i < 4; i++) {
		loc_rand_nb[i] = raw_data[14 + i];
	}
	loc_TA = 
		(raw_data[18] << 24) +
		(raw_data[19] << 16) +
		(raw_data[20] << 8) +
		(raw_data[21]);
	loc_gain_TX =
		(raw_data[22] << 24) +
		(raw_data[23] << 16) +
		(raw_data[24] << 8) +
		(raw_data[25]);
	loc_gain_RX =
		(raw_data[26] << 24) +
		(raw_data[27] << 16) +
		(raw_data[28] << 8) +
		(raw_data[29]);
	posX_constel = 
		(raw_data[30] << 8) +
		(raw_data[31]);
	posY_constel =
		(raw_data[32] << 8) +
		(raw_data[33]);
	posX_eye =
		(raw_data[34] << 8) +
		(raw_data[35]);
	posY_eye =
		(raw_data[36] << 8) +
		(raw_data[37]);
	/*
	offset_freqTX_tuned = 
		(raw_data[38] << 8) +
		(raw_data[39]);
	offset_freqTX_conf =
		(raw_data[40] << 8) +
		(raw_data[41]);
	offset_freqRX_tuned =
		(raw_data[42] << 8) +
		(raw_data[43]);
	offset_freqRX_conf =
		(raw_data[44] << 8) +
		(raw_data[45]);
	*/
	if (match == 0) { config_OK = 0; }
	if ((loc_TA > 1500000) || (loc_TA < -10000)) { config_OK = 0; }
	if ((loc_gain_TX > 200) || (loc_gain_TX < -200)) { config_OK = 0; }
	if ((loc_gain_RX > 200) || (loc_gain_RX < -200)) { config_OK = 0; }
	printf("READ NVRAM : \r\n");
	printf("   match: %i\r\n", match);
	printf("   rand_nb: %02X:%02X:%02X:%02X\r\n", loc_rand_nb[0], loc_rand_nb[1], loc_rand_nb[2], loc_rand_nb[3]);
	printf("   TA saved: %i\r\n", loc_TA);
	printf("   TX gain: %i\r\n", loc_gain_TX);
	printf("   RX gain: %i\r\n", loc_gain_RX);
	printf("   Config OK: %i\r\n", config_OK);
	printf("   Window Constellation: %i %i\r\n", posX_constel, posY_constel);
	printf("   Window Eye: %i %i\r\n", posX_eye, posY_eye);
	//printf("   Freq_offset conf RX:%i TX:%i ; tun RX:%i TX:%i\r\n", offset_freqRX_conf, offset_freqTX_conf, offset_freqRX_tuned, offset_freqTX_tuned);
	if (config_OK == 0) {
		printf("NVRAM : no previous entry\r\n");
		NPRV_init_NVRAM();
		NPRV_write_NVRAM();
	}
	else {
		CONF_modem_MAC[4] = loc_rand_nb[2];
		CONF_modem_MAC[5] = loc_rand_nb[3];
		for (i = 0; i < 4; i++) {
			G_NPRV_my_random_nb[i] = loc_rand_nb[i];
		}
		G_NPRV_my_TimeAdvance = loc_TA;
		NPRV_current_TX_gain = loc_gain_TX;
		NPRV_current_RX_gain = loc_gain_RX;
		if (posX_constel < 1600) { NVRAM_posX_constel = posX_constel; }
		if (posY_constel < 1200) { NVRAM_posY_constel = posY_constel; }
		if (posX_eye < 1600) {NVRAM_posX_eye = posX_eye;}
		if (posY_eye < 1600) { NVRAM_posY_eye = posY_eye; }
	}
	if (fid_in != 0) {
		fclose(fid_in);
	}
	NPRV_display_set_windows_pos(NVRAM_posX_constel, NVRAM_posY_constel, NVRAM_posX_eye, NVRAM_posY_eye);
	//NPRV_SDR_init_freq_offset_NVRAM(offset_freqRX_conf, offset_freqTX_conf, offset_freqRX_tuned, offset_freqTX_tuned);
	return 1;
}

int NPRV_write_NVRAM(void) {
	FILE* fid_out;
	uint8_t raw_data[60] = { "NPRV_nvram001:" };//14 bytes
	int i;
	int32_t loc_gain_RX, loc_gain_TX, loc_TA;
	int16_t offset_freqTX_tuned, offset_freqTX_conf, offset_freqRX_tuned, offset_freqRX_conf;
	//NPRV_SDR_get_freq_offset_for_NVRAM(&offset_freqRX_conf, &offset_freqTX_conf, &offset_freqRX_tuned, &offset_freqTX_tuned);
	fid_out = fopen("prog_data/NPRVSAT.nvm", "wb");
	if (fid_out == 0) {
		printf("ERROR dir ./prog_data/ do not exist.\r\n");
		return -1;
	}
	for (i = 0; i < 4; i++) {
		raw_data[14 + i] = G_NPRV_my_random_nb[i];
	}
	if ((G_NPRV_my_TimeAdvance < 800000) && (G_NPRV_my_TimeAdvance > 0)) {
		loc_TA = G_NPRV_my_TimeAdvance;
	}
	else {
		loc_TA = NPRV_NVRAM_TA_INIT;
	}
	raw_data[18] = (loc_TA & 0xFF000000) >> 24;
	raw_data[19] = (loc_TA & 0x00FF0000) >> 16;
	raw_data[20] = (loc_TA & 0x0000FF00) >> 8;
	raw_data[21] = (loc_TA & 0x000000FF);
	
	loc_gain_TX = NPRV_current_TX_gain;//converts to int
	//!!! Ajouter des conditions de validit
	raw_data[22] = (loc_gain_TX & 0xFF000000) >> 24;
	raw_data[23] = (loc_gain_TX & 0x00FF0000) >> 16;
	raw_data[24] = (loc_gain_TX & 0x0000FF00) >> 8;
	raw_data[25] = (loc_gain_TX & 0x000000FF);
	
	loc_gain_RX = NPRV_current_RX_gain;
	//!!! Ajouter des conditions de validit
	raw_data[26] = (loc_gain_RX & 0xFF000000) >> 24;
	raw_data[27] = (loc_gain_RX & 0x00FF0000) >> 16;
	raw_data[28] = (loc_gain_RX & 0x0000FF00) >> 8;
	raw_data[29] = (loc_gain_RX & 0x000000FF);

	raw_data[30] = (NVRAM_posX_constel & 0xFF00) >> 8;
	raw_data[31] = (NVRAM_posX_constel & 0x00FF);

	raw_data[32] = (NVRAM_posY_constel & 0xFF00) >> 8;
	raw_data[33] = (NVRAM_posY_constel & 0x00FF);

	raw_data[34] = (NVRAM_posX_eye & 0xFF00) >> 8;
	raw_data[35] = (NVRAM_posX_eye & 0x00FF);

	raw_data[36] = (NVRAM_posY_eye & 0xFF00) >> 8;
	raw_data[37] = (NVRAM_posY_eye & 0x00FF);
	/*
	raw_data[38] = (offset_freqTX_tuned & 0xFF00) >> 8;
	raw_data[39] = (offset_freqTX_tuned & 0x00FF);

	raw_data[40] = (offset_freqTX_conf & 0xFF00) >> 8;
	raw_data[41] = (offset_freqTX_conf & 0x00FF);

	raw_data[42] = (offset_freqRX_tuned & 0xFF00) >> 8;
	raw_data[43] = (offset_freqRX_tuned & 0x00FF);

	raw_data[44] = (offset_freqRX_conf & 0xFF00) >> 8;
	raw_data[45] = (offset_freqRX_conf & 0x00FF);
	*/
	for (i = 18; i < 22; i++) { printf("%02X:", raw_data[i]); }
	printf("\r\n");
	fwrite(raw_data, sizeof(uint8_t), 38, fid_out);//30 38
	fclose(fid_out);
	printf("WRITE NVRAM : \r\n");
	printf("   rand_nb: %02X:%02X:%02X:%02X\r\n", G_NPRV_my_random_nb[0], G_NPRV_my_random_nb[1], G_NPRV_my_random_nb[2], G_NPRV_my_random_nb[3]);
	printf("   TA saved: %i\r\n", loc_TA);
	printf("   TX gain: %i\r\n", loc_gain_TX);
	printf("   RX gain: %i\r\n", loc_gain_RX);
	printf("   Window Constellation: %i %i\r\n", NVRAM_posX_constel, NVRAM_posY_constel);
	printf("   Window Eye: %i %i\r\n", NVRAM_posX_eye, NVRAM_posY_eye);
	//printf("   Freq_offset conf RX:%i TX:%i ; tun RX:%i TX:%i\r\n", offset_freqRX_conf, offset_freqTX_conf, offset_freqRX_tuned, offset_freqTX_tuned);
	printf("Write NVRAM OK\r\n");
	return 1;
}

void NPRV_save_wind_pos(int posX_constel, int posY_constel, int constel_opened, int posX_eye, int posY_eye, int eye_opened) {
	if (constel_opened == 1) {
		NVRAM_posX_constel = posX_constel;
		NVRAM_posY_constel = posY_constel;
	}
	if (eye_opened == 1) {
		NVRAM_posX_eye = posX_eye;
		NVRAM_posY_eye = posY_eye;
	}
}