// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#include "pch.h"
#include <SFML/System.hpp>
#include "NPRV_TDMA.h"
#include "NPRV_L1_L2_radio.h"
#include "NPRV_glob_variables.h"
#include "Eth_IPv4.h"
#include "NPRV_coder_decoder.h"
#include "NPRV_SDR_if.h"
#include "NPRV_sat_emul.h"
#include "NPRV_config.h"

#ifdef NPRV_linux
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#endif

#pragma warning(disable: 4996)

//NPRV_MAX_CLIENT_NB

//#define NPRV_TDMA_ALLOC_DEBUG
#define NPRV_NEW_TDMA_ALLOC

sf::Mutex mutex_TDMA_measure_TA;
sf::Mutex mutex_TDMA_uplink_info;

static uint32_t TDMA_my_TX_burst_start[NPRV_MAX_CHANNEL_COUNT][32] = { 0 };// [channel][burst_counter]

const int TDMA_glob_chan_std = 1;	// 1:100kS !!! incompatible with a mix of 50k & 100k carriers at the same time
									// 2: 50kS
static uint32_t TDMA_last_Master_synchro_T = 1000000;
static unsigned char TDMA_last_Master_synchro_SCounter = 0;


void NPRV_TDMA_plan_discovery_TX(int channel, uint32_t T_begin, uint32_t slot_duration, unsigned char uslot_counter) {
	int rand_loc;
	uint32_t T_begin_real, slot_duration_real, my_frame_duration, begin_offset, begin_offset_max;
	uint32_t rand_glob, big_TA_margin;
	rand_glob =
		(G_NPRV_my_random_nb[0] << 24) +
		(G_NPRV_my_random_nb[1] << 16) +
		(G_NPRV_my_random_nb[2] << 8) +
		(G_NPRV_my_random_nb[3]);
	my_frame_duration = NPRV_compute_slot_duration(channel, 32);
	//big_TA_margin = (CONF_TDMA_Frame_slot_nb * CONF_slot_duration) / 4;
	big_TA_margin = slot_duration / 4; 
	//begin_offset_max = slot_duration - (8 * CONF_slot_duration) - my_frame_duration; // (2 * CONF_slot_duration) //8
	begin_offset_max = slot_duration - (2 * big_TA_margin) - my_frame_duration; // (2 * CONF_slot_duration) //8
	rand_loc = rand();
	rand_loc = rand_loc + rand_glob;
	//begin_offset = (4 * CONF_slot_duration) + (rand_loc % begin_offset_max);//(1 * CONF_slot_duration)//4
	begin_offset = big_TA_margin + (rand_loc % begin_offset_max);//(1 * CONF_slot_duration)//4
	T_begin_real = T_begin + begin_offset;
	//printf("\r\nDISCOVERY TX PLAN: Duration:%u T:%u offset_max:%u offset_real:%u\r\n", my_frame_duration, T_begin_real, begin_offset_max, begin_offset);
	//NPRV_HMI_new_TXT_message();
	//NPRV_emul_HMI_new_TXT_message();
	NPRV_tx_plan_new_burst(channel, T_begin_real, (my_frame_duration + 5000), uslot_counter);
	//exit(-1);
}

void NPRV_client_interpret_TDMA_alloc(unsigned char* raw_frame, int size) {
	int index;
	unsigned int TX_ID, dest_ID;
	unsigned int current_ID; 
	unsigned char current_chan;
	unsigned char current_duration;
	unsigned char current_uslot;
	uint32_t T_next_burst;
	uint32_t duration_us;
	uint32_t margain = 2000;
	int stop_entry_detected = 0;
	unsigned char* data_field;
	data_field = raw_frame + 8;
	index = 11;
	TX_ID = ((raw_frame[3] & 0x1F) << 8) + raw_frame[4];
	dest_ID = ((data_field[0] & 0x1F) << 8) + data_field[1];
	//printf("TDMA ALLOC RX\r\n");
	if ((CONF_is_NPRV_HUB == 0) && (TX_ID == 0) && (dest_ID == 0x1FFF)) {//only for clients
		mutex_plan_new_burst.lock();
		while ((index < (size - 3)) && (stop_entry_detected == 0)) {
			current_ID = ((raw_frame[index] & 0xFF) << 5) +
				((raw_frame[index + 1] & 0xF8) >> 3);
			current_chan = ((raw_frame[index + 1] & 0x03) << 3) +
				((raw_frame[index + 2] & 0xE0) >> 5);
			if (current_chan >= NPRV_MAX_CHANNEL_COUNT) { current_chan = NPRV_MAX_CHANNEL_COUNT; }
			current_duration = (raw_frame[index + 2] & 0x1F);
			current_uslot = raw_frame[index + 3];
			index = index + 4;
			if (current_ID == 0x1FFF) {
				//printf("final TDMA entry\r\n");
				stop_entry_detected = 1;
			}
			//if (current_chan >= CONF_NPRV_channel_count) {
			//	printf("TDMA Alloc chelou! ID%u chan%u duration:%u slot:%u\r\n", current_ID, current_chan, current_duration, current_uslot);
			//}
			if ( (current_ID == G_NPRV_MyClientID)  ) {//&& (current_chan < CONF_NPRV_channel_count)
				if (NPRV_channels_SR_std[current_chan] == 0) { margain = CONF_slot_T_margin; } //margain = 1000; 
				if (NPRV_channels_SR_std[current_chan] == 1) { margain = CONF_slot_T_margin * 2; } //margain = 2000; 
				if (NPRV_channels_SR_std[current_chan] == 2) { margain = CONF_slot_T_margin / 2; } // margain = 500; 
				//printf("TDMA Alloc: ID%u chan%u duration:%u slot:%u\r\n", current_ID, current_chan, current_duration, current_uslot);
				//NPRV_HMI_new_TXT_message();
				T_next_burst = TDMA_get_T_future_slot (current_uslot) - G_NPRV_my_TimeAdvance + CONF_T_offset_channel * current_chan;
				duration_us = (current_duration * CONF_slot_duration) - margain;//1000
				if (current_duration == CONF_TDMA_Frame_slot_nb) {// additional margin in case of a full slot
					duration_us = duration_us - (CONF_slot_duration / 2);
				}
				if (G_NPRV_MyClientID == 0x1FFE) {
					NPRV_TDMA_plan_discovery_TX(current_chan, T_next_burst, duration_us, current_uslot);
				}
				else {
					NPRV_tx_plan_new_burst(current_chan, T_next_burst, duration_us, current_uslot);
				}
			}
		}
		mutex_plan_new_burst.unlock();
	}
}

void TDMA_TX_burst_event_inform(unsigned char channel, unsigned char burst_counter, uint32_t TX_time) {
	unsigned char loc_burst_counter;
	loc_burst_counter = burst_counter & 0x0F;
	TDMA_my_TX_burst_start[channel][loc_burst_counter] = TX_time;
}



void TDMA_measure_my_TA(unsigned char announced_chan, unsigned char announced_burst_count, uint32_t loc_RX_time_sync) {
	uint32_t loc_TX_time;
	uint32_t loc_RX_time_preamb;
	int32_t delta_T;
	//if (CONF_is_NPRV_HUB == 0) { mutex_TDMA_measure_TA.lock(); }
	loc_TX_time = TDMA_my_TX_burst_start[announced_chan][announced_burst_count];
	if (NPRV_channels_SR_std[announced_chan] == 0) {
		loc_RX_time_preamb = loc_RX_time_sync - ((CONF_NPRV_preamb_size_long + 2) * 80); //160: 10 sample_per_symb x 8 symb_per_byte
	}
	if (NPRV_channels_SR_std[announced_chan] == 1) {
		loc_RX_time_preamb = loc_RX_time_sync - ((CONF_NPRV_preamb_size_long + 2) * 160); //160: 20 sample_per_symb x 8 symb_per_byte
	}
	if (NPRV_channels_SR_std[announced_chan] == 2) {
		loc_RX_time_preamb = loc_RX_time_sync - ((CONF_NPRV_preamb_size_long + 2) * 40); //40: 5 sample_per_symb x 8 symb_per_byte
	}

	delta_T = (int32_t)loc_RX_time_preamb - (int32_t)loc_TX_time;
	//printf("TX_time:%lu RX_time:%lu deltaT:%lu chan:%u\r\n", loc_TX_time, loc_RX_time_preamb, delta_T, announced_chan);
	G_NPRV_my_TimeAdvance = delta_T;
	//if (CONF_is_NPRV_HUB == 0) { mutex_TDMA_measure_TA.unlock(); }
}

void TDMA_measure_my_TA_discovery(unsigned char announced_chan, unsigned char announced_burst_count, uint32_t loc_RX_time_sync, uint32_t rand_RX) {
	uint32_t my_rand_nb;
	my_rand_nb =
		(G_NPRV_my_random_nb[0] << 24) +
		(G_NPRV_my_random_nb[1] << 16) +
		(G_NPRV_my_random_nb[2] << 8) +
		(G_NPRV_my_random_nb[3]);
	if (rand_RX == my_rand_nb) {
		TDMA_measure_my_TA(announced_chan, announced_burst_count, loc_RX_time_sync);
		printf("MY DISCOVERY rand:%08X\r\n", rand_RX);
		NPRV_HMI_new_TXT_message(); 
		NPRV_emul_HMI_new_TXT_message();
	}
}

void TDMA_measure_Master_timings(unsigned char TDMA_byte, int top_synchro, unsigned int TX_ID, uint32_t timing, unsigned char channel) {
	uint32_t loc_RX_time_preamb = 0;
	if (NPRV_channels_SR_std[channel] == 0) {
		loc_RX_time_preamb = timing - ((CONF_NPRV_preamb_size_long + 2) * 80);
	}
	if (NPRV_channels_SR_std[channel] == 1) {
		loc_RX_time_preamb = timing - ((CONF_NPRV_preamb_size_long + 2) * 160);
	}
	if (NPRV_channels_SR_std[channel] == 2) {
		loc_RX_time_preamb = timing - ((CONF_NPRV_preamb_size_long + 2) * 40);
	}
	if ((CONF_is_NPRV_HUB == 0) && (top_synchro == 1) && (TX_ID == 0)) {
		mutex_TDMA_measure_TA.lock();
		TDMA_last_Master_synchro_SCounter = TDMA_byte;
		TDMA_last_Master_synchro_T = loc_RX_time_preamb - CONF_T_offset_channel * channel;
		mutex_TDMA_measure_TA.unlock();
		//printf("** Syncro RX client; slot:%u T:%u\r\n", TDMA_last_Master_synchro_SCounter, TDMA_last_Master_synchro_T);
	}
}


uint32_t TDMA_get_T_future_slot(unsigned char slot_nb) {
	uint32_t answer;
	int delta_T;
	if (CONF_is_NPRV_HUB == 0) { mutex_TDMA_measure_TA.lock(); }
	delta_T = (slot_nb - TDMA_last_Master_synchro_SCounter) & 0xFF;
	delta_T = delta_T * CONF_slot_duration;
	//printf("delta T:%i\r\n", delta_T);
	if (CONF_is_NPRV_HUB == 0) { mutex_TDMA_measure_TA.unlock(); }
	answer = TDMA_last_Master_synchro_T + delta_T;
	return answer;
}

unsigned char NPRV_elaborate_client_TDMA_byte(void) {
	int IPv4_filling;
	int slot_count;
	unsigned char TDMA_byte;
	IPv4_filling = NPRV_IPv4_get_TX_filling();
	slot_count = IPv4_filling / 184;//125
	if (slot_count > 250) { slot_count = 250; }
	TDMA_byte = slot_count;
	//printf("TDMA byte client:%u IPv4_filling%i\r\n",TDMA_byte, IPv4_filling);
	return TDMA_byte;
}