// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#include "pch.h"
#include "NPRV_glob_variables.h"

#ifdef NPRV_SFML
#include "SFML/System.hpp"
sf::Clock G_clock;
//sf::Mutex mutex_frame_interp;
sf::Mutex mutex_plan_new_burst;
#endif

int G_NPRV_RX_radio_frame_count[NPRV_MAX_CHANNEL_COUNT] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
int32_t G_NPRV_my_TimeAdvance = 220000;

long unsigned int NPRV_G_rx_macro_timer[NPRV_MAX_CHANNEL_COUNT] = { 0 };

int CONF_TX_test = 0;
int CONF_TXTEST_length = 200;
int CONF_TXTEST_salve_nb = 3;
int CONF_is_NPRV_HUB = 0;
int CONF_RX_SAVE_to_FILE = 0;
//int CONF_Frame_SAVE_to_FILE = 0;
int CONF_Emul_Radio = 0;
uint32_t CONF_Emul_Radio_IP = 0;
#ifdef NPRV_FEC_7_4
uint32_t CONF_slot_duration = 14500;	//with reference 100kS/s
#else
uint32_t CONF_slot_duration = 23800;	//with reference 100kS/s previously 28000
#endif

uint32_t CONF_slot_T_margin = 1200;		//with reference 100kS/s previously 1200
uint32_t CONF_TDMA_Frame_slot_nb = 16;	//with reference 100kS/s previously 10
uint32_t CONF_T_offset_channel = 1 * 2493;//2493

int CONF_radio_addr_TXChanCapab[NPRV_MAX_CLIENT_NB] = { 1,0 };//,0,0,0,1,0,1,0,0 
int CONF_radio_addr_TXGlobCapab[NPRV_MAX_CLIENT_NB] = { 3,0 };
int CONF_radio_addr_RXChanCapab[NPRV_MAX_CLIENT_NB] = { 2,0 };//,0,0,0,2,0,2,0,0 
int CONF_my_RFGlob_TX_capab = 1;
int CONF_my_RFChan_TX_capab = 1;
int CONF_my_radio_RX_capab = 2;
int CONF_SDR_HW_type = 0;
char CONF_SDR_PLUTO_IP[20] = "192.168.2.1";
int CONF_INHIB_TUNTAP = 0;
int CONF_IPv4_SPY = 0;
int CONF_HUB_NULL_TX = 0;
int CONF_NPRV_LOG_FILE = 0;

int32_t CONF_Maxi_Gain_TX_user = 200;
int32_t CONF_Hub_TX_PWR = 0;
int CONF_TX_filter_active = 1;
int CONF_force_BPSK = 0;
float CONF_center_freq_TX = 1245e6;// 1245e6;
float CONF_center_freq_RX = 435e6;// 435e6;
float NPRV_RX_LO_freq = 0;

float NPRV_freq_offset_RX_tuned = 0;
float NPRV_freq_offset_TX_tuned = 0;

float NPRV_current_TX_gain = 29;
float NPRV_current_RX_gain = 27;//29

uint32_t CONF_radio_addr_table_IP_begin[NPRV_MAX_CLIENT_NB] = { 0xC0A80001, 0 };//0, 0, 0, 0xC0A80078, 0, 0xC0A80064
uint32_t CONF_radio_addr_table_IP_size[NPRV_MAX_CLIENT_NB] = { 0x00000050, 0 };//, 0, 0, 0, 0x00000004, 0, 0x00000004 
unsigned char CONF_DHCP_route_table[100] = { 0 };
int CONF_radio_addr_table_status[NPRV_MAX_CLIENT_NB] = { 1, 0 };//, 0, 0, 0, 1, 0, 1

char CONF_client_callsigns_table_hub[NPRV_MAX_CLIENT_NB][18] = { 0 };
unsigned char CONF_client_RandNb_table_hub[NPRV_MAX_CLIENT_NB][6] = { 0 };
uint32_t CONF_radio_addr_table_date[NPRV_MAX_CLIENT_NB] = { 0 };

char G_NPRV_my_Callsign[20] = { 0 }; 
char G_NPRV_tuntap_name[20] = { 0 };

unsigned char G_NPRV_my_random_nb[6] = { 0 };
uint32_t CONF_my_IP_size_requested;
int CONF_my_IP_static_requested;
unsigned int G_NPRV_MyClientID = 0x323;
unsigned int G_NPRV_MyRadioState = 0;
unsigned int G_NPRV_MyIPConfigured = 0;
//float G_NPRV_my_amplitude_client = 0;//migre sdr if
//float G_NPRV_amplitude_ref_hub = 0;//migre sdr if
//int G_NPRV_RX_gain_stabilised = 0;//migre sdr if
uint32_t NPRV_G_tx_timer = 0;
int TX_curr_status[NPRV_MAX_CHANNEL_COUNT] = { 0, 0, 0, 0, 0 };			//0: no TX
													//1: next TX pending
													//2: ramp up
													//3: TX in progress
													//4: ramp down

unsigned char CONF_modem_MAC[10] = { 0x4E, 0x50, 0x52, 0x56, 0xA1, 0xA2 };
uint32_t CONF_radio_IP_start = 0xC0A80064;//64
uint32_t CONF_radio_IP_size = 0x28; // for Master
int CONF_Hub_Max_IP_per_client = 2;

//float GLOB_ampl_stat_per_ID[NPRV_MAX_CLIENT_NB] = { 0 };//migre sdr if

int CONF_NPRV_channel_count = 2;
int CONF_NPRV_filter_TX = 1;
int CONF_NPRV_preamble_typ = 1;
int CONF_NPRV_preamb_size_long = 32;
int CONF_NPRV_preamb_size_short = 16;//!!! should be 8 ; modified 2022 09 18
int CONF_NPRV_filter_RX = 1;
int CONF_NPRV_insert_QPSK_symb = 0;

//float NPRV_MaxAmpliRX_AGC_temp[10] = { 0 };
float NPRV_MaxAmpliRX_AGC[NPRV_MAX_CHANNEL_COUNT] = { 0 };

//float NPRV_channels_freq[NPRV_MAX_CHANNEL_COUNT] = {
//	+0.100, //+0.125 130 180
//	-0.100, //-0.125
//	-0.300, //-0.375
//	+0.300	//+0.375
//};

float NPRV_channels_freq[NPRV_MAX_CHANNEL_COUNT] = {
	+0.125, //+0.125 
	-0.125, //-0.125
	-0.375, //-0.375
	+0.375,	//+0.375
	-0.380	//+0.375
};

int NPRV_channels_SR_std[NPRV_MAX_CHANNEL_COUNT] = {// 0:100kS
	0,												// 1: 50kS
	1,												// 2:200kS
	0,												// 3: 25kS
	0,
};

int NPRV_chan_config_valid = 0;
int NPRV_IPgen_config_valid = 0;
int NPRV_CONF_filter_chan_zero = 1;
//unsigned char TDMA_table_uplink_needs[NPRV_MAX_CLIENT_NB] = { 0 };



char RX_who_callsigns[NPRV_MAX_CLIENT_NB][18] = { 0 };
int RX_who_static_IP[NPRV_MAX_CLIENT_NB] = { 0 };
uint32_t RX_who_IP_range_start[NPRV_MAX_CLIENT_NB] = { 0 };
uint32_t RX_who_IP_range_size[NPRV_MAX_CLIENT_NB] = { 0 };

//config provided from Hub to Client
uint32_t NPRV_CONF_client_modem_IP = 0xFFFFFFFF;
uint32_t NPRV_CONF_hub_modem_IP = 0xFFFFFFFF;
uint32_t NPRV_CONF_DNS_IP = 0xFFFFFFFF;
int	NPRV_CONF_DNS_active_hub = 0;
int	NPRV_CONF_DNS_active_client = 0;
int NPRV_DNS_activ_from_Hub = 0;
uint32_t NPRV_CONF_subnet = 0xFFFFFF00;

int NPRV_my_IP_static = 0;

LAN_conf_T LAN_conf_applied = {
	0xC0A800FE, /* modem IP 10.0.44.254 */
	0xC0A80064, /* DHCP range start 64 */
	0x00000004, /* DHCP range size */
	0xFFFFFF00, /* subnet mask */
	0,			/* def route active */
	0,			/* LAN_DNS_activ */
	0,			/* LAN_DNS_value */
	1			/* DHCP_server_active */
};

int G_TX_rframe_count = 0;