// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#ifndef NPRVSAT_GLOB_VARIABLES
#define NPRVSAT_GLOB_VARIABLES

#define NPRV_SFML
#include <SFML/System.hpp>

#ifdef __linux__
#define NPRV_linux
#else
#define NPRV_windows
#endif

#define NPRV_SW_VERSION "2023_02_01"

//#define NPR_70
#define NPR_VSAT

#define NPRV_MAX_CLIENT_NB 128
#define NPRV_MAX_CHANNEL_COUNT 20

#define NPRV_BPSK 0
#define NPRV_QPSK 2

#define NPRV_WITH_HUB_CODE

#define NPRV_MULTITHREAD

#define NPRV_WITH_OUTER_FEC_RS
#define NPRV_WITH_HEADER_FEC_RS

//#define NPRV_CONFIG_QPSK_PARAM
//#define NPRV_CONFIG_DEBUG

//#define NPRV_CODER_WITH_TEST_CODE
//#define NPRV_WITH_FEC13_8_CODE
//#define NPRV_NEW_INTERLEAVING
//#define NPRV_FEC_7_4

//#define NPRV_WITH_FECTEST_CODE

//#define NPRV_WITH_4FSK_CODE
#define NPRV_WITH_QPSK_CODE

//#define NPRV_SDR_DATA_FROM_FILE
//#define NPRV_SIGNAL_FAST_TIMING_FOR_DEBUG

extern sf::Clock G_clock;

//extern sf::Mutex mutex_frame_interp;
extern sf::Mutex mutex_plan_new_burst;
//extern sf::Mutex mutex_tuntap_TX;

extern int G_NPRV_RX_radio_frame_count[NPRV_MAX_CHANNEL_COUNT];
extern int32_t G_NPRV_my_TimeAdvance;

extern long unsigned int NPRV_G_rx_macro_timer[NPRV_MAX_CHANNEL_COUNT];

extern char G_NPRV_tuntap_name[20];
extern char G_NPRV_my_Callsign[20];
extern unsigned char G_NPRV_my_random_nb[6];
extern uint32_t CONF_my_IP_size_requested;
extern int CONF_my_IP_static_requested;
extern unsigned int G_NPRV_MyClientID;
extern unsigned int G_NPRV_MyRadioState;
extern unsigned int G_NPRV_MyIPConfigured;
//extern float G_NPRV_my_amplitude_client;
//extern float G_NPRV_amplitude_ref_hub;
//extern int G_NPRV_RX_gain_stabilised;
//extern long unsigned int NPRV_G_tx_timer;
extern uint32_t NPRV_G_tx_timer;
extern int TX_curr_status[NPRV_MAX_CHANNEL_COUNT];// = { 0, 0, 0, 0, 0  //0: no TX
													//1: next TX pending
													//2: ramp up
													//3: TX in progress
													//4: ramp down
extern int CONF_TX_test;
extern int CONF_TXTEST_length;
extern int CONF_TXTEST_salve_nb ;
extern int CONF_is_NPRV_HUB; 
extern int CONF_RX_SAVE_to_FILE; 
//extern int CONF_Frame_SAVE_to_FILE;
extern int CONF_Emul_Radio;
extern uint32_t CONF_Emul_Radio_IP;

extern uint32_t CONF_slot_duration; 
extern uint32_t CONF_slot_T_margin;
extern uint32_t CONF_TDMA_Frame_slot_nb;
extern uint32_t CONF_T_offset_channel;

//#define NPRV_CLIENT_ADDR_SIZE 256


extern int CONF_radio_addr_TXChanCapab[NPRV_MAX_CLIENT_NB];//capability in simultaneous channel nb
extern int CONF_radio_addr_TXGlobCapab[NPRV_MAX_CLIENT_NB];//capability in simultaneous channel nb
extern int CONF_radio_addr_RXChanCapab[NPRV_MAX_CLIENT_NB];
extern int CONF_my_RFGlob_TX_capab;
extern int CONF_my_RFChan_TX_capab;
extern int CONF_my_radio_RX_capab;
extern int CONF_SDR_HW_type;
extern char CONF_SDR_PLUTO_IP[20];
extern int CONF_INHIB_TUNTAP;
extern int CONF_IPv4_SPY;
extern int CONF_HUB_NULL_TX;
extern int CONF_NPRV_LOG_FILE;

extern int32_t CONF_Maxi_Gain_TX_user;
extern int32_t CONF_Hub_TX_PWR;
extern int CONF_TX_filter_active;
extern int CONF_force_BPSK;
extern float CONF_center_freq_TX;
extern float CONF_center_freq_RX;
extern float NPRV_RX_LO_freq;

extern float NPRV_freq_offset_RX_tuned;
extern float NPRV_freq_offset_TX_tuned;

extern float NPRV_current_TX_gain;
extern float NPRV_current_RX_gain;

extern uint32_t CONF_radio_addr_table_IP_begin[NPRV_MAX_CLIENT_NB];
extern uint32_t CONF_radio_addr_table_IP_size[NPRV_MAX_CLIENT_NB];
extern unsigned char CONF_DHCP_route_table[100];
extern int CONF_radio_addr_table_status[NPRV_MAX_CLIENT_NB];
extern char CONF_client_callsigns_table_hub[NPRV_MAX_CLIENT_NB][18];
extern unsigned char CONF_client_RandNb_table_hub[NPRV_MAX_CLIENT_NB][6];
extern uint32_t CONF_radio_addr_table_date[NPRV_MAX_CLIENT_NB];

extern int CONF_NPRV_channel_count;
extern int CONF_NPRV_filter_TX;
extern int CONF_NPRV_preamble_typ;
//#define PREAMB_SIZE_BYTE_LONG 32
//#define PREAMB_SIZE_BYTE_SHORT 8 /*8*/
extern int CONF_NPRV_preamb_size_long;
extern int CONF_NPRV_preamb_size_short;
extern int CONF_NPRV_filter_RX;
extern int CONF_NPRV_insert_QPSK_symb;
//extern float NPRV_MaxAmpliRX_AGC[10][10];//[step][chan]
//extern float NPRV_MaxAmpliRX_AGC_temp[10];
extern float NPRV_MaxAmpliRX_AGC[NPRV_MAX_CHANNEL_COUNT];

extern unsigned char CONF_modem_MAC[10];
extern uint32_t CONF_radio_IP_start;
extern uint32_t CONF_radio_IP_size;
extern int CONF_Hub_Max_IP_per_client;

//extern float GLOB_ampl_stat_per_ID[NPRV_MAX_CLIENT_NB];

//extern unsigned char TDMA_table_uplink_needs[NPRV_MAX_CLIENT_NB];

extern float NPRV_channels_freq[NPRV_MAX_CHANNEL_COUNT];
extern int NPRV_channels_SR_std[NPRV_MAX_CHANNEL_COUNT];
extern int NPRV_chan_config_valid;
extern int NPRV_IPgen_config_valid;
extern int NPRV_CONF_filter_chan_zero;

extern char RX_who_callsigns[NPRV_MAX_CLIENT_NB][18];
extern int RX_who_static_IP[NPRV_MAX_CLIENT_NB];
extern uint32_t RX_who_IP_range_start[NPRV_MAX_CLIENT_NB];
extern uint32_t RX_who_IP_range_size[NPRV_MAX_CLIENT_NB];

//config provided from Hub to Client
extern uint32_t NPRV_CONF_client_modem_IP;
extern uint32_t NPRV_CONF_hub_modem_IP;
extern uint32_t NPRV_CONF_DNS_IP;
extern int	NPRV_CONF_DNS_active_hub;
extern int	NPRV_CONF_DNS_active_client;
extern int NPRV_DNS_activ_from_Hub;
extern uint32_t NPRV_CONF_subnet;

extern int NPRV_my_IP_static;

struct LAN_conf_T {

	//unsigned char modem_MAC[6]; 
	uint32_t LAN_modem_IP;
	uint32_t DHCP_range_start;
	uint32_t DHCP_range_size; //4
	uint32_t LAN_subnet_mask;
	uint32_t LAN_def_route;
	unsigned char LAN_def_route_activ;
	unsigned char LAN_DNS_activ;
	uint32_t LAN_DNS_value;
	unsigned char DHCP_server_active;
};

extern LAN_conf_T LAN_conf_applied;

extern int G_TX_rframe_count;

#endif