// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#include "pch.h"
#include "NPRV_glob_variables.h"
#include "NPRV_save_to_file.h"
#include "Eth_IPv4.h"

#ifdef NPRV_linux
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#endif

#pragma warning(disable: 4996)

int CONF_Frame_SAVE_to_FILE = 0;
static FILE* file_frames_capture;

void NPRV_init_save_frame_to_file(int config) {
	CONF_Frame_SAVE_to_FILE = config;
	if (CONF_Frame_SAVE_to_FILE == 1) {
		file_frames_capture = fopen("user_data/Frame_capture.csv", "w+");
		fprintf(file_frames_capture, "Time,channel,top,modulation,length,TDMA_byte,protocol,TX_ID,Dest_ID,burst_count,slot_count,ann_channel,pkt_count,seg_count\r\n");
		printf("save to file frame\r\n");
	}
}

void NPRV_frame_save2file(
	uint32_t time, unsigned char channel, int top, int modulation, unsigned int size, 
	unsigned char TDMA_byte, unsigned char protocol, unsigned int TX_ID, unsigned int dest_ID, 
	unsigned char burst_count, unsigned char slot_count, unsigned char ann_channel, 
	unsigned char pkt_count, unsigned char seg_count, 
	unsigned char* raw_frame, int size2)
{
	int i;
	if (CONF_Frame_SAVE_to_FILE == 1) {
		fprintf(file_frames_capture, "%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,%u,", time, channel, top, modulation, size, TDMA_byte,protocol,
			TX_ID, dest_ID, burst_count, slot_count, ann_channel, pkt_count, seg_count);
		for (i = 0; i < size2; i++) {
			fprintf(file_frames_capture, "%02X.", raw_frame[i]);
		}
		fprintf(file_frames_capture,"\r\n");
		
	}
}

void NPRV_save_exit(void) {
	if (CONF_Frame_SAVE_to_FILE == 1) {
		fclose(file_frames_capture);
	}
}