#include <math.h>
#include <stdio.h>
#include <stdint.h>
#include <cstdint>
#include <stdlib.h>     /* srand, rand */
#include <string.h>
#include "NPRV_ReedSolomon_24.h"

#pragma warning(disable: 4996)

#define NO_PRINT

#define maxrand 0x7fffffff
#define mm  8           /* RS code over GF(2**mm) - change to suit */
#define n   256   	/* n = size of the field */
#define nn  255         /* nn=2**mm -1   length of codeword */
#define tt  12          /* number of errors that can be corrected 16 */
#define kk  231         /* kk = nn-2*tt 223 */ /* Degree of g(x) = 2*tt */

/**** Primitive polynomial ****/
int pp_RS_24 [mm+1] = { 1, 0, 1, 1, 1, 0, 0, 0, 1}; /* 1+x^2+x^3+x^4+x^8 */

/* int pp [mm+1] = { 1, 1, 0, 0, 1}; */ /* 1 + x + x^4 */

/* int pp[mm+1] = {1,1,0,1}; */ /* 1 + x + x^3 */

/* generator polynomial, tables for Galois field */
int alpha_to_RS24 [nn+1], index_of_RS24 [nn+1], gg_RS_24 [nn-kk+1];
int b0_RS_24;  /* g(x) has roots @**b0, @**(b0+1), ... ,@^(b0+2*tt-1) */

/* data[] is the info vector, bb[] is the parity vector, recd[] is the 
  noise corrupted received vector  */
int recd_RS_24[nn], data_RS_24[kk], bb_RS_24[nn-kk] ;

void generate_gf_24()
 {
   register int i, mask ;

  mask = 1 ;
  alpha_to_RS24[mm] = 0 ;
  for (i=0; i<mm; i++)
   { alpha_to_RS24[i] = mask ;
     index_of_RS24[alpha_to_RS24[i]] = i ;
     if (pp_RS_24[i]!=0) /* If pp[i] == 1 then, term @^i occurs in poly-repr of @^mm */
       alpha_to_RS24[mm] ^= mask ;  /* Bit-wise EXOR operation */
     mask <<= 1 ; /* single left-shift */
   }
  index_of_RS24[alpha_to_RS24[mm]] = mm ;
  /* Have obtained poly-repr of @^mm. Poly-repr of @^(i+1) is given by 
     poly-repr of @^i shifted left one-bit and accounting for any @^mm 
     term that may occur when poly-repr of @^i is shifted. */
  mask >>= 1 ;
  for (i=mm+1; i<nn; i++)
   { if (alpha_to_RS24[i-1] >= mask)
        alpha_to_RS24[i] = alpha_to_RS24[mm] ^ ((alpha_to_RS24[i-1]^mask)<<1) ;
     else alpha_to_RS24[i] = alpha_to_RS24[i-1]<<1 ;
     index_of_RS24[alpha_to_RS24[i]] = i ;
   }
  index_of_RS24[0] = -1 ;
 }


void gen_poly_24()
/* Obtain the generator polynomial of the tt-error correcting, length
  nn=(2**mm -1) Reed Solomon code from the product of (X+@**(b0+i)), i = 0, ... ,(2*tt-1)
  Examples: 	If b0 = 1, tt = 1. deg(g(x)) = 2*tt = 2.
 	g(x) = (x+@) (x+@**2)
		If b0 = 0, tt = 2. deg(g(x)) = 2*tt = 4.
	g(x) = (x+1) (x+@) (x+@**2) (x+@**3)  	
*/
 {
   register int i,j ;

   gg_RS_24[0] = alpha_to_RS24[b0_RS_24];
   gg_RS_24[1] = 1 ;    /* g(x) = (X+@**b0) initially */
   for (i=2; i <= nn-kk; i++)
    { gg_RS_24[i] = 1 ;
      /* Below multiply (gg[0]+gg[1]*x + ... +gg[i]x^i) by (@**(b0+i-1) + x) */
      for (j=i-1; j>0; j--)
        if (gg_RS_24[j] != 0)  gg_RS_24[j] = gg_RS_24[j-1]^ alpha_to_RS24[((index_of_RS24[gg_RS_24[j]])+b0_RS_24+i-1)%nn] ;
        else gg_RS_24[j] = gg_RS_24[j-1] ;
      gg_RS_24[0] = alpha_to_RS24[((index_of_RS24[gg_RS_24[0]])+b0_RS_24+i-1)%nn] ;     /* gg[0] can never be zero */
    }
   /* convert gg[] to index form for quicker encoding */
   for (i=0; i <= nn-kk; i++)  gg_RS_24[i] = index_of_RS24[gg_RS_24[i]] ;
 }

void NPRV_init_ReedSolomon_24(void) {
	int i;
	generate_gf_24();
	b0_RS_24 = 19;
	gen_poly_24();
	//printf("GG for %i:\r\n", tt * 2);
	//for (i = 0; i <= nn - kk; i++) {
	//	printf("%02X:", gg_RS_24[i]);
	//}
	//printf("\r\n");
}

void encode_ReedSolomon_24_loc(void)
/* take the string of symbols in data[i], i=0..(k-1) and encode systematically
   to produce 2*tt parity symbols in bb[0]..bb[2*tt-1]
   data[] is input and bb[] is output in polynomial form.
   Encoding is done by using a feedback shift register with appropriate
   connections specified by the elements of gg[], which was generated above.
   Codeword is   c(X) = data(X)*X**(nn-kk)+ b(X)          */
 {
   register int i,j ;
   int feedback ;

   for (i=0; i<nn-kk; i++)   bb_RS_24[i] = 0 ;
   for (i=kk-1; i>=0; i--)
    {  feedback = index_of_RS24[data_RS_24[i]^bb_RS_24[nn-kk-1]] ;
       if (feedback != -1) /* feedback term is non-zero */
        { for (j=nn-kk-1; j>0; j--)
            if (gg_RS_24[j] != -1)
              bb_RS_24[j] = bb_RS_24[j-1]^alpha_to_RS24[(gg_RS_24[j]+feedback)%nn] ;
            else
              bb_RS_24[j] = bb_RS_24[j-1] ;
          bb_RS_24[0] = alpha_to_RS24[(gg_RS_24[0]+feedback)%nn] ;
        }
       else /* feedback term is zero. encoder becomes a single-byte shifter */
        { for (j=nn-kk-1; j>0; j--)
            bb_RS_24[j] = bb_RS_24[j-1] ;
          bb_RS_24[0] = 0 ;
        } ;
    } ;
 }

int NPRV_encode_ReedSolomon_24(uint8_t* data_in, int length_in) {
	// Code word = 64
	// data field = 191
	int i, start_pos, parity_len;
	parity_len = 2 * tt;
	start_pos = kk - length_in;
	for (i = 0; i < start_pos; i++) {
		data_RS_24[i] = 0; // padding
	}
	for (i = 0; i < length_in; i++) {
		data_RS_24[i + start_pos] = data_in[i];
	}
	encode_ReedSolomon_24_loc();
	for (i = 0; i < parity_len; i++) {
		data_in[length_in + i] = bb_RS_24[i];
	}
	return (length_in + parity_len);
}

/* Performs ERRORS-ONLY decoding of RS codes. If decoding is successful,
 writes the codeword into recd[] itself. Otherwise recd[] is unaltered. 
 If channel caused no more than "tt" errors, the tranmitted codeword will
 be recovered.
*/ 

int decode_ReedSolomon_24_loc(void)
 {
   register int i,j,u,q ;
   int elp[nn-kk+2][nn-kk], d[nn-kk+2], l[nn-kk+2], u_lu[nn-kk+2], s[nn-kk+1] ;
   int count=0, syn_error=0, root[tt], loc[tt], z[tt+1], err[nn], reg[tt+1] ;

/* recd[] is in polynomial form, convert to index form */
   for (i=0;i < nn;i++) recd_RS_24[i] = index_of_RS24[recd_RS_24[i]];

/* first form the syndromes; i.e., evaluate recd(x) at roots of g(x) namely
 @**(b0+i), i = 0, ... ,(2*tt-1) */
   for (i=1; i <= nn-kk; i++)
    { s[i] = 0 ;
      for (j=0; j < nn; j++)
        if (recd_RS_24[j] != -1)
          s[i] ^= alpha_to_RS24[(recd_RS_24[j]+(b0_RS_24+i-1)*j)%nn] ;      /* recd[j] in index form */
/* convert syndrome from polynomial form to index form  */
      if (s[i] != 0)  syn_error = 1 ;   /* set flag if non-zero syndrome => error */
      s[i] = index_of_RS24[s[i]] ;
    };

   if (syn_error)       /* if errors, try and correct */
    {
/* initialise table entries */
      d[0] = 0 ;           /* index form */
      d[1] = s[1] ;        /* index form */
      elp[0][0] = 0 ;      /* index form */
      elp[1][0] = 1 ;      /* polynomial form */
      for (i=1; i<nn-kk; i++)
        { elp[0][i] = -1 ;   /* index form */
          elp[1][i] = 0 ;   /* polynomial form */
        }
      l[0] = 0 ;
      l[1] = 0 ;
      u_lu[0] = -1 ;
      u_lu[1] = 0 ;
      u = 0 ;

      do
      {
        u++ ;
        if (d[u]==-1)
          { l[u+1] = l[u] ;
            for (i=0; i<=l[u]; i++)
             {  elp[u+1][i] = elp[u][i] ;
                elp[u][i] = index_of_RS24[elp[u][i]] ;
             }
          }
        else
/* search for words with greatest u_lu[q] for which d[q]!=0 */
          { q = u-1 ;
            while ((d[q]==-1) && (q>0)) q-- ;
/* have found first non-zero d[q]  */
            if (q>0)
             { j=q ;
               do
               { j-- ;
                 if ((d[j]!=-1) && (u_lu[q]<u_lu[j]))
                   q = j ;
               }while (j>0) ;
             } ;

/* have now found q such that d[u]!=0 and u_lu[q] is maximum */
/* store degree of new elp polynomial */
            if (l[u]>l[q]+u-q)  l[u+1] = l[u] ;
            else  l[u+1] = l[q]+u-q ;

/* form new elp(x) */
            for (i=0; i<nn-kk; i++)    elp[u+1][i] = 0 ;
            for (i=0; i<=l[q]; i++)
              if (elp[q][i]!=-1)
                elp[u+1][i+u-q] = alpha_to_RS24[(d[u]+nn-d[q]+elp[q][i])%nn] ;
            for (i=0; i<=l[u]; i++)
              { elp[u+1][i] ^= elp[u][i] ;
                elp[u][i] = index_of_RS24[elp[u][i]] ;  /*convert old elp value to index*/
              }
          }
        u_lu[u+1] = u-l[u+1] ;

/* form (u+1)th discrepancy */
        if (u<nn-kk)    /* no discrepancy computed on last iteration */
          {
            if (s[u+1]!=-1)
                   d[u+1] = alpha_to_RS24[s[u+1]] ;
            else
              d[u+1] = 0 ;
            for (i=1; i<=l[u+1]; i++)
              if ((s[u+1-i]!=-1) && (elp[u+1][i]!=0))
                d[u+1] ^= alpha_to_RS24[(s[u+1-i]+index_of_RS24[elp[u+1][i]])%nn] ;
            d[u+1] = index_of_RS24[d[u+1]] ;    /* put d[u+1] into index form */
          }
      } while ((u < nn-kk) && (l[u+1] <= tt)) ;


      u++ ;
      if (l[u] <= tt)         /* can correct error */
       {
/* put elp into index form */
         for (i=0; i<=l[u]; i++)   elp[u][i] = index_of_RS24[elp[u][i]] ;
/* find roots of the error location polynomial */
         for (i=1; i <= l[u]; i++)
           reg[i] = elp[u][i] ;
         count = 0 ;
         for (i=1; i <= nn; i++)
          {  q = 1 ;
             for (j=1; j<=l[u]; j++)
              if (reg[j]!=-1)
                { reg[j] = (reg[j]+j)%nn ;
                  q ^= alpha_to_RS24[reg[j]] ;
                } ;
             if (!q)        /* store root and error location number indices */
              { root[count] = i;
                loc[count] = nn-i ;
                count++ ;
              };
          } ;
#ifndef NO_PRINT
	  printf("Computed Error Locations\n");
	  for (i=0;i < l[u];i++) printf("%d ",loc[i]);
	  printf("\n");
#endif
         if (count == l[u])    /* no. roots = degree of elp hence <= tt errors */
          {
/* form polynomial z(x) */
           for (i=1; i <= l[u]; i++)        /* Z[0] = 1 always - do not need */
            { if ((s[i]!=-1) && (elp[u][i]!=-1))
                 z[i] = alpha_to_RS24[s[i]] ^ alpha_to_RS24[elp[u][i]] ;
              else if ((s[i]!=-1) && (elp[u][i]==-1))
                      z[i] = alpha_to_RS24[s[i]] ;
                   else if ((s[i]==-1) && (elp[u][i]!=-1))
                          z[i] = alpha_to_RS24[elp[u][i]] ;
                        else
                          z[i] = 0 ;
              for (j=1; j<i; j++)
                if ((s[j]!=-1) && (elp[u][i-j]!=-1))
                   z[i] ^= alpha_to_RS24[(elp[u][i-j] + s[j])%nn] ;
              z[i] = index_of_RS24[z[i]] ;         /* put into index form */
            } ;
  /* evaluate errors at locations given by error location numbers loc[i] */
           for (i=0; i<nn; i++)
             { err[i] = 0 ;
               if (recd_RS_24[i]!=-1)        /* convert recd[] to polynomial form */
                 recd_RS_24[i] = alpha_to_RS24[recd_RS_24[i]] ;
               else  recd_RS_24[i] = 0 ;
             }
           for (i=0; i < l[u]; i++)    /* compute numerator of error term first */
            { err[loc[i]] = 1;       /* accounts for z[0] */
              for (j=1; j<=l[u]; j++){
                if (z[j]!=-1)
                  err[loc[i]] ^= alpha_to_RS24[(z[j]+j*root[i])%nn] ;
	      } /* z(x) evaluated at X(l)**(-1) */
              if (err[loc[i]] != 0) /* term X(l)**(1-b0) */
		 err[loc[i]] = alpha_to_RS24[(index_of_RS24[err[loc[i]]]+root[i]*(b0_RS_24+nn-1))%nn];
              if (err[loc[i]]!=0)
               {
		 err[loc[i]] = index_of_RS24[err[loc[i]]] ;
                 q = 0 ;     /* form denominator of error term */
                 for (j=0; j<l[u]; j++)
                   if (j!=i)
                     q += index_of_RS24[1^alpha_to_RS24[(loc[j]+root[i])%nn]] ;
                 q = q % nn ;
                 err[loc[i]] = alpha_to_RS24[(err[loc[i]]-q+nn)%nn] ;
                 recd_RS_24[loc[i]] ^= err[loc[i]] ;  /*recd[i] must be in polynomial form */
               }
            }
	    return(1);
          }
          else{    /* no. roots != degree of elp => >tt errors and cannot solve */
           	for (i=0; i<nn; i++){        /* could return error flag if desired */
               	   if (recd_RS_24[i]!=-1)        /* convert recd[] to polynomial form */
                 	recd_RS_24[i] = alpha_to_RS24[recd_RS_24[i]] ;
                   else  
			recd_RS_24[i] = 0 ;     /* just output received word as is */
		}
		return(2);
	  }
       }
      else{         /* elp has degree has degree >tt hence cannot solve */
       	for (i=0; i<nn; i++){       /* could return error flag if desired */
            if (recd_RS_24[i]!=-1)        /* convert recd[] to polynomial form */
                recd_RS_24[i] = alpha_to_RS24[recd_RS_24[i]] ;
            else  
		recd_RS_24[i] = 0 ;     /* just output received word as is */
	}
	return(3);
      }
   }
   else{       /* no non-zero syndromes => no errors: output received codeword */
       	for (i=0; i < nn; i++){
       	   if (recd_RS_24[i] != -1)        /* convert recd[] to polynomial form */
         	recd_RS_24[i] = alpha_to_RS24[recd_RS_24[i]] ;
           else  
	 	recd_RS_24[i] = 0 ;
 	}
	return(0);
   }
 }

 int NPRV_decode_ReedSolomon_24(uint8_t* data_loc, int length_in) {
	 int answer;
	 int i, start_pos, parity_len, length_ext;
	 parity_len = 2 * tt;
	 start_pos = kk - length_in;
	 length_ext = length_in + parity_len;
	 for (i = 0; i < start_pos; i++) {
		 recd_RS_24[i] = 0; // padding
	 }
	 for (i = 0; i < length_ext; i++) {
		 recd_RS_24[i + start_pos] = data_loc[i];
	 }

	 answer = decode_ReedSolomon_24_loc();
	 for (i = 0; i < length_in; i++) {
		 data_loc[i] = recd_RS_24[i + start_pos];
	 }

	 return answer;
 }
