﻿// This file is part of "NPR-VSAT SDR Modem" software
// (New-Packet-Radio, via VSAT, software modem for PC with duplex SDR)
// (A virtual/SDR data IPv4 modem for amateur radio geostationary satellite, 
// Mostly designed for Hamnet connectivity over QO-100)
// Copyright (c) 2020-2023 Guillaume F. F4HDK (amateur radio callsign)
// contact e-mail : f4hdk [at] free [dot] fr
// 
// "NPR-VSAT Software Modem" is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// "NPR-VSAT Software Modem" is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with "NPR-VSAT Software Modem".  If not, see <http://www.gnu.org/licenses/>

#include "pch.h"
#include "NPRV_glob_variables.h"

#include <stdio.h>

#include <fcntl.h>

#ifdef NPRV_windows
#include <iostream>
//#include<functional>
#include<windows.h>
#include<winioctl.h>
#include <string.h>
#include <io.h>
#include "Win_tuntap_if.h"
#pragma warning(disable: 4996)
#endif

#ifdef NPRV_linux
#include "Linux_tuntap_if.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#endif

#include <SFML/System.hpp>
#include "Eth_IPv4.h"
#include "DHCP_ARP.h"
#include "NPRV_SDR_if.h"
#include "NPRV_config.h"
#include "NPRV_sat_emul.h"
#include "NPRV_TDMA.h"
#include "NPRV_NVRAM.h"
#include "NPRV_coder_decoder.h"
#include "Reed_Solomon/NPRV_ReedSolomon.h"
#include "NPRV_log_file.h"


//int main() {
int main(int argc, char* argv[]) {
	unsigned char packet[20000] = { 0 };
	int RX_size;
	int i;
	int count_loc = 0;
	uint32_t rand_nb_loc;
	float freq_loc = 0;
	int ans_loc = 0;
	//printf("0:hello world 115\r\n"); 
	printf("NPR-VSAT (by F4HDK)\r\nSW version: %s\r\n", NPRV_SW_VERSION);
	printf("BINARY for Client only\r\n");
	srand(time(NULL));
#ifdef NPRV_windows
	//_setmode(_fileno(stdout), _O_U16TEXT);
#endif
#ifdef NPRV_linux
	//setlocale(LC_ALL, ""); 
#endif

	NPRV_RedSolomon_init();
	NPRV_init_config();
	NPRV_read_config_from_file(1);//1 = all data
	if (CONF_is_NPRV_HUB == 0) {
		NPRV_init_client_config();
	}
	
	NPRV_display_config();
	NPRV_read_NVRAM();
	
	
	NPR_IPv4_init_RX_reconstr();
	printf("main with %i arg\r\n", argc);
	for (i = 1; i < argc; i++) {
		printf("ARG %i \"%s\"\r\n", i, argv[i]);
	}
	if (argc >= 3) {
		if (strcmp(argv[1], "TX_test") == 0) {
			//printf("ON FAIT TX TEST\r\n");
			ans_loc = sscanf(argv[2], "%f", &freq_loc);
			printf("FREQ:%i %f\r\n", ans_loc, freq_loc);
			if (ans_loc == 1) {
				NPRV_config_save_TX_test_RX_freq(freq_loc * 1e6);
				CONF_TX_test = 1;
				CONF_center_freq_TX = freq_loc * 1e6;
				CONF_center_freq_RX = freq_loc * 1e6 - NPRV_RX_LO_freq;
				int table_conversion_chan_loc[10] = { 1, 1, 1, 0, 2, 2 };
				G_NPRV_MyClientID = 0x1FFA;
				CONF_NPRV_channel_count = 1;// remove if we want to test multiple chanels
				NPRV_channels_SR_std[0] = table_conversion_chan_loc[CONF_my_RFGlob_TX_capab];

				printf("\r\n\r\nNEW SR:%i\r\n\r\n", NPRV_channels_SR_std[0]);
			}
		}
	}
	if (argc >= 2) {
		if (strcmp(argv[1], "reset_TX_gain") == 0) {
			NPRV_SDR_init_gain(1);
			printf("** GAIN TX RESET to Max ** %f\r\n", NPRV_current_TX_gain);
		}
	}
	if (CONF_INHIB_TUNTAP == 0) {
		init_tuntap(); //!!!
	}
	if ((CONF_is_NPRV_HUB == 0) && (CONF_IPv4_SPY == 2)) {
		connect_tuntap();
	}
	int j;
	reset_DHCP_table(); //!!!

	if (CONF_Emul_Radio == 1) {
		NPRV_sat_emul_glob_loop();
	} 
	else {
		NPRV_SDR_main_loop();
	}
	printf("\r\nend main\r\n");
	sf::sleep(sf::milliseconds(50));
	return 1;
}
